#include <dbus/dbus-glib.h>
#include <stdio.h>
#include <stdlib.h>

int
main (int argc, char **argv)
{
  DBusGConnection *bus;
  DBusGProxy *remote_object_introspectable;
  GError *error = NULL;
  gchar *introspect_data;
  gint k, n;

  g_type_init ();

  {
    GLogLevelFlags fatal_mask;
    
    fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
    fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
    g_log_set_always_fatal (fatal_mask);
  }

  bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
  	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Couldn't connect to session bus: %s", error->message);
	exit (EXIT_FAILURE);
  }

  remote_object_introspectable = dbus_g_proxy_new_for_name (bus,
							    "org.beepmediaplayer.SystemControl",
							    "/org/beepmediaplayer/SystemControl/BmpSystemControl",
							    "org.freedesktop.DBus.Introspectable");
  if (!dbus_g_proxy_call (remote_object_introspectable, "Introspect", &error,
			  G_TYPE_INVALID,
			  G_TYPE_STRING, &introspect_data, G_TYPE_INVALID))
    {
	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to complete Introspect: %s", error->message);
	exit (EXIT_FAILURE);
    }
  printf ("%s", introspect_data);
  g_free (introspect_data);
 
  exit(EXIT_SUCCESS);
}
