#include <dbus/dbus-glib.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

int
main (int argc, char **argv)
{
  DBusGConnection *bus;
  DBusGProxy *remote_object;
  GError *error = NULL;
  GValueArray *array_lists;
  GValueArray *array_list;
  const gchar *listname, *filename;
  gchar *aux;
  gint k, n;

  g_type_init ();

  {
    GLogLevelFlags fatal_mask;
    
    fatal_mask = g_log_set_always_fatal (G_LOG_FATAL_MASK);
    fatal_mask |= G_LOG_LEVEL_WARNING | G_LOG_LEVEL_CRITICAL;
    g_log_set_always_fatal (fatal_mask);
  }

  bus = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
  if (!bus)
  {
  	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Couldn't connect to session bus: %s", error->message);
	exit (EXIT_FAILURE);
  }
  
  remote_object = dbus_g_proxy_new_for_name (bus,
					     "org.beepmediaplayer.SystemControl",
					     "/org/beepmediaplayer/SystemControl/BmpSystemControl",
					     "org.beepmediaplayer.SystemControl");

  if (!dbus_g_proxy_call (remote_object, "GetPlaylists", &error,
			  G_TYPE_INVALID,
			  G_TYPE_VALUE_ARRAY, &array_lists, G_TYPE_INVALID))
  {
  	g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "failed to complete getplaylists: %s", error->message);
	exit(EXIT_FAILURE);
  }

  printf ("<?xml version='1.0'?>\n");
  printf ("<bmp:playlists>\n\n");

  for (k = 0; k < array_lists->n_values; k++)
  {
	listname = g_value_get_string((const GValue*)g_value_array_get_nth(array_lists,k));
	printf("\t<bmp:playlist name='%s'>\n", listname); 

        if (!dbus_g_proxy_call (remote_object, "GetPlaylist", &error,
			  G_TYPE_STRING, listname, 
			  G_TYPE_INVALID,
			  G_TYPE_VALUE_ARRAY, 
			  &array_list, 
			  G_TYPE_INVALID))
	{
		g_log (G_LOG_DOMAIN, G_LOG_LEVEL_ERROR, "Failed to complete GetPlaylist: %s", error->message);
	}

	for (n = 0; n < array_list->n_values; n++)
	{
		filename = g_value_get_string((const GValue*)g_value_array_get_nth(array_list,n));
		aux = g_markup_escape_text(filename, strlen(filename));
		printf ("\t\t<bmp:playlist-file position='%d'>%s</playlist-file>\n", n, aux);
		g_free(aux);
	}
	printf("\t</bmp:playlist>\n");
  }
  printf("\n</bmp:playlists>\n\n");
 
  exit(EXIT_SUCCESS);
}
