#ifndef METADATA_H
#define METADATA_H

#include <gtk/gtk.h>

typedef enum {
    BMP_DATUM_TITLE,
    BMP_DATUM_TIME,
    BMP_DATUM_TRACK,
    BMP_DATUM_RATING,
    BMP_DATUM_ARTIST,
    BMP_DATUM_ALBUM,
    BMP_DATUM_DATE,
    BMP_DATUM_GENRE,
    BMP_DATUM_COMMENT,
    BMP_DATUM_BITRATE,
    BMP_DATUM_LOCATION,
    BMP_DATUM_MTIME,
    BMP_DATUM_NONE
} BmpTagId;

typedef struct bmp_query_atom bmp_query_atom_t;
struct bmp_query_atom {
    BmpTagId  tag_id;
    gchar    *match_value;
    gboolean  exact_match;
};

typedef struct bmp_query bmp_query_t;
struct bmp_query {
    GList    *query_atoms;
    GList    *result_ufid_list;
    gboolean  do_sort;
    BmpTagId  sort_tag_id;

    /* Private field */
    gpointer  user_data;
};

GHashTable *
bmp_metadata_get_from_uri (const gchar * filename);

void
bmp_metadata_get_for_files_threaded (BmpListType *list, gint start, gint end);

const gchar*
bmp_metadata_get_metadata_field (GHashTable *metadata, BmpTagId tag_id);

gchar*
bmp_metadata_get_titlestring (GHashTable *metadata);
 
gchar*
bmp_metadata_get_title (BmpTagId tag_id);

gchar*
bmp_metadata_get_id (BmpTagId tag_id);

const gchar*
bmp_metadata_get_id_static (BmpTagId tag_id);

G_BEGIN_DECLS

/* Metadata Cache Functions */
#define BMP_TYPE_METADATA_CACHE		  (bmp_metadata_cache_get_type ())
#define BMP_METADATA_CACHE(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_METADATA_CACHE, BmpMetadataCache))
#define BMP_METADATA_CACHE_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_METADATA_CACHE, BmpMetadataCacheClass))
#define BMP_IS_METADATA_CACHE(obj)	  (GTK_CHECK_TYPE ((obj), BMP_TYPE_METADATA_CACHE))
#define BMP_IS_METADATA_CACHE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_METADATA_CACHE))
#define BMP_METADATA_CACHE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_METADATA_CACHE, BmpMetadataCacheClass))

typedef struct _BmpMetadataCache BmpMetadataCache;
typedef struct _BmpMetadataCacheClass BmpMetadataCacheClass;
typedef struct _BmpMetadataCachePrivate BmpMetadataCachePrivate;

struct _BmpMetadataCache {
	GObject parent;

        BmpMetadataCachePrivate *private;
};

struct _BmpMetadataCacheClass {
	GObjectClass parent;
};

GType
bmp_metadata_cache_get_type (void);

/* Creation */
BmpMetadataCache*
bmp_metadata_cache_new (void);

/* Save/Restore to/from storage */
void
bmp_metadata_cache_write (BmpMetadataCache *self);
void
bmp_metadata_cache_read (BmpMetadataCache *self);

/* Metadata acquisition */
GHashTable*
bmp_metadata_cache_get_metadata_for_uri (BmpMetadataCache *self, const gchar *path);

GHashTable* 
bmp_metadata_cache_get_metadata_for_ufid (BmpMetadataCache *self, gint ufid);

GtkListStore*
bmp_metadata_cache_list_tag_id (BmpMetadataCache *self, BmpTagId tag_id);
 
/* Query API */
bmp_query_t*
bmp_metadata_cache_query_construct (BmpMetadataCache *self, gboolean do_sort, BmpTagId sort_tag_id, gint n_atoms, ...);

void
bmp_metadata_cache_query_run (BmpMetadataCache *self, bmp_query_t *query);

void
bmp_metadata_cache_query_dispose (BmpMetadataCache *self, bmp_query_t *query);

#endif /* METADATA_H */

