#ifndef _BMP_PLAY_H
#define _BMP_PLAY_H

#include <config.h>

#include <glib-object.h>
#define BMP_TYPE_PLAY		  (bmp_play_get_type ())
#define PLAY(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_PLAY, BmpPlay))
#define PLAY_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_PLAY, BmpPlayClass))
#define BMP_IS_PLAY(obj)	  (G_TYPE_CHECK_INSTANCE_TYPE ((obj), BMP_TYPE_PLAY))
#define BMP_IS_PLAY_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_PLAY))
#define PLAY_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_PLAY, BmpPlayClass))

typedef struct _BmpPlay BmpPlay;
typedef struct _BmpPlayClass BmpPlayClass;
typedef struct _BmpPlayPrivate BmpPlayPrivate;

struct _BmpPlay {
    GObject parent;
    /*
     * instance members 
     */
    BmpPlayPrivate *private;
};

struct _BmpPlayClass {
    GObjectClass parent;

    void (*eos)         (BmpPlay * self);
    void (*error)       (BmpPlay * self);
    void (*buffer)      (BmpPlay * self);
    void (*no_sound)    (BmpPlay * self);
    void (*time_tick)   (BmpPlay * self, gint length, gint position);
};

typedef enum {
	PLAYSTATUS_STOPPED = 1 << 1,
	PLAYSTATUS_PLAYING = 1 << 2,
	PLAYSTATUS_PAUSED  = 1 << 3,
	PLAYSTATUS_SEEKING = 1 << 4,
	PLAYSTATUS_WAITING = 1 << 5
} BmpPlayStatus;

#ifdef HAVE_XINE
#define BMP_EQ_TO_ENGINE_OFFSET 18
#endif

typedef enum {
    BMP_EQ_30HZ,
    BMP_EQ_60HZ,
    BMP_EQ_125HZ,
    BMP_EQ_250HZ,
    BMP_EQ_500HZ,
    BMP_EQ_1000HZ,
    BMP_EQ_2000HZ,
    BMP_EQ_4000HZ,
    BMP_EQ_8000HZ,
    BMP_EQ_16000HZ,
    
    BMP_N_EQ_PARAMS
} BmpEqEnum;

/*
 * used by BMP_TYPE_PLAY 
 */
GType bmp_play_get_type(void);

/*
 * API 
 */
BmpPlay*
bmp_play_new(void);

#ifdef HAVE_XINE
void
bmp_play_get_metadata_xine (BmpPlay *self, GHashTable *metadata, const gchar * filename);

void
bmp_play_set_eq (BmpPlay *self, BmpEqEnum eq, gint value);

gchar**
bmp_play_get_cd (BmpPlay * self, gint *n_mrls);
 
gboolean
bmp_play_stop_stream_fade (BmpPlay * self);
#endif

#ifdef HAVE_GST
GHashTable*
bmp_play_get_metadata_gst (BmpPlay *self, const gchar * filename);

gboolean
bmp_play_prepare(BmpPlay * self);

gboolean
bmp_play_ready (BmpPlay * self);
#endif

gboolean
bmp_play_set_stream(BmpPlay * self, const gchar * uri);

gboolean
bmp_play_change_stream(BmpPlay * self, const gchar * uri);

gboolean
bmp_play_play_stream(BmpPlay * self);

gboolean
bmp_play_pause_stream(BmpPlay * self);

gboolean
bmp_play_stop_stream (BmpPlay * play);

gboolean
bmp_play_seek_stream(BmpPlay * self, gint offset);

gint
bmp_play_get_length(BmpPlay * self);

gint
bmp_play_get_time(BmpPlay * self);

gint
bmp_play_get_status(BmpPlay * self);

void
bmp_play_set_volume(BmpPlay * self, gint volume);

#ifdef HAVE_GST
void
bmp_play_probe_outsinks(BmpPlay * play);
#endif

#endif                          /* PLAY_H */
