#include <gtk/gtk.h>
#include <glib/gstdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include <main.h>

#include <bmp_playlist.h>
#include <bmp_metadata.h>

#include <g_object_get.h>
#include <util.h>
#include <uri.h>

G_DEFINE_TYPE (BmpPlaylist, bmp_playlist, G_TYPE_OBJECT)

typedef enum {
        BMP_PLAYLIST_SIGNAL_LIST_DELETED,
        BMP_PLAYLIST_SIGNAL_LIST_ADDED,
        BMP_PLAYLIST_SIGNAL_LIST_RENAMED,

        BMP_PLAYLIST_SIGNAL_TRACKLIST_LIST_SORTED,
	BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_REMOVED,
	BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_ADDED,

	BMP_PLAYLIST_SIGNAL_SET_CURRENT_LIST,

	BMP_PLAYLIST_N_SIGNALS
} BmpPlaylistSignals;

static guint signals[BMP_PLAYLIST_N_SIGNALS] = { 0, 0, 0, 0, 0, 0, 0 };

typedef enum {
	BMP_PLAYLIST_PROP_0,

	BMP_PLAYLIST_N_PROPERTIES
} BmpPlaylistProperties;

struct _BmpPlaylistPrivate
{
        gboolean dispose_has_run;

	BmpListType *playlists;
	ListManager *list_manager;

	gchar *pre_current_name;
	gint pre_current_position;
};

/* Track List funcs */
gchar*
tracklist_column_data_func (BmpFileItem *item, gint column_idx)
{
    gchar *response = NULL;

#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	switch (column_idx)
	{
		case 0:
		{
			if (item->metadata) {
				response = bmp_metadata_get_titlestring (item->metadata);
			} else {
				response = g_path_get_basename (item->uri);
			}
		}
		break;

		case 1:
		{

			gint ttime;
			gint minutes;
			gint seconds;
			const gchar *time;

			if (!item->metadata)
                            return g_strdup("");

			time = bmp_metadata_get_metadata_field(item->metadata, BMP_DATUM_TIME);

			if (time) {
			    ttime = strtol (time,NULL,10);
			}
			else
			    ttime = 0;

			minutes = ttime / 60;
			seconds = ttime % 60;

			response = g_strdup_printf("%d:%-2.2d",minutes,seconds);
		}
		break;
	}

	return response;
}

static void
tracklist_item_swap_func (GList *items, gint _idx1, gint _idx2)
{
	BmpFileItem *fi_a, *fi_b;

 	GList *a, *b;

	a = g_list_nth (items, _idx1);
	b = g_list_nth (items, _idx2);

	glist_swap(a,b);

	fi_a = a->data;
	fi_b = b->data;

	fi_a->dirty = TRUE;
	fi_b->dirty = TRUE;
}


void
tracklist_on_display_func (BmpFileItem *item)
{
	if (!item) return;
	if (item->metadata_attempted) return;
	if (item->metadata != NULL) return;

	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }

	item->metadata = bmp_metadata_get_from_uri (item->uri);
	item->metadata_attempted = TRUE;
	item->dirty = TRUE;
}

static gboolean
tracklist_get_dirty_func (BmpFileItem *item)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	if (!item) return FALSE;
	return item->dirty;
}

static void
tracklist_set_dirty_func (BmpFileItem *item, gboolean dirty)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	if (!item) return;
	item->dirty = dirty;
}

static gboolean
tracklist_get_rendered_func (BmpFileItem *item)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	if (!item) return FALSE;
	return item->rendered;
}

static void
tracklist_set_rendered_func (BmpFileItem *item, gboolean rendered)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	if (!item) return;
	item->rendered = rendered;
}


static gboolean
tracklist_get_selected_func (BmpFileItem *item)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	return item->selected;
}

static void
tracklist_set_selected_func (BmpFileItem *item, gboolean selected)
{
#if 0
	while (item->locked) { while (gtk_events_pending()) gtk_main_iteration(); }
#endif

	item->selected = selected;
	item->dirty = TRUE;
}

static void
tracklist_item_destroy_func (BmpFileItem *item)
{
    g_free (item->uri);
    g_free (item);
}

/* Playlists List funcs */
static gchar*
playlists_column_data_func (BmpPlaylistItem *item, gint column_idx)
{
	if (!item) return NULL;

	return item->listname;
}

static gboolean
playlists_get_dirty_func (BmpPlaylistItem *item)
{
	if (!item) return FALSE;
	return item->dirty;
}

static void
playlists_set_dirty_func (BmpPlaylistItem *item, gboolean dirty)
{
	if (!item) return;
	item->dirty = dirty;
}

static gboolean
playlists_get_rendered_func (BmpPlaylistItem *item)
{
	if (!item) return FALSE;
	return item->dirty;
}

static void
playlists_set_rendered_func (BmpPlaylistItem *item, gboolean dirty)
{
	if (!item) return;
	item->dirty = dirty;
}


static gboolean
playlists_get_selected_func (BmpPlaylistItem *item)
{
	if (!item) return FALSE;

	return item->selected;
}

static void
playlists_set_selected_func (BmpPlaylistItem *item, gboolean selected)
{
	item->selected = selected;
	item->dirty = TRUE;
}

static void
playlists_item_destroy_func (BmpPlaylistItem *item)
{
	/*g_free (item->filename);
	g_free (item->listname);*/
}

static gint
playlists_compare_func (const BmpPlaylistItem *a, const BmpPlaylistItem *b)
{
	gchar *string_a;
	gchar *string_b;
	gint cmp_result;

	string_a = g_utf8_casefold (a->listname, strlen(a->listname));
	string_b = g_utf8_casefold (b->listname, strlen(b->listname));

	if ( (string_a != NULL) && (string_b == NULL)) return  1;
	if ( (string_a == NULL) && (string_b == NULL)) return  0;
	if ( (string_a == NULL) && (string_b != NULL)) return -1;

	cmp_result = g_utf8_collate (string_a, string_b);

	/*g_free (string_a);
	g_free (string_b);*/

	return cmp_result;
}

static gint
tracklist_compare_func (BmpFileItem *a, BmpFileItem *b, gpointer data)
{
	BmpTagId tag_id;
	gchar *cmptext_a, *cmptext_b;
	gchar *string_a;
	gchar *string_b;
	gint cmp_result;
        gint val_a, val_b;

	tracklist_on_display_func (a);
	tracklist_on_display_func (b);

	if (! data)
	{
		cmptext_a = tracklist_column_data_func (a, 0);
		cmptext_b = tracklist_column_data_func (b, 0);

		if ( (cmptext_a != NULL) && (cmptext_b == NULL)) return  1;
		if ( (cmptext_a == NULL) && (cmptext_b == NULL)) return  0;
		if ( (cmptext_a == NULL) && (cmptext_b != NULL)) return -1;

	}
	else
	{
		tag_id = GPOINTER_TO_INT(data);

		cmptext_a = g_hash_table_lookup(a->metadata, bmp_metadata_get_id_static(tag_id));
		cmptext_b = g_hash_table_lookup(b->metadata, bmp_metadata_get_id_static(tag_id));

		if ( (cmptext_a != NULL) && (cmptext_b == NULL)) return  1;
		if ( (cmptext_a == NULL) && (cmptext_b == NULL)) return  0;
		if ( (cmptext_a == NULL) && (cmptext_b != NULL)) return -1;

		if ((tag_id == BMP_DATUM_TRACK) || (tag_id == BMP_DATUM_DATE))
		{
			if (! cmptext_a)
			    val_a = 0;
			else
			    val_a = strtol(cmptext_a,NULL,10);

			if (! cmptext_b)
			    val_b = 0;
			else
			    val_b = strtol(cmptext_b,NULL,10);

			return val_a - val_b;
		}
	}

	string_a = g_utf8_casefold (cmptext_a, strlen(cmptext_a));
	string_b = g_utf8_casefold (cmptext_b, strlen(cmptext_b));

	if ( (string_a != NULL) && (string_b == NULL)) return  1;
	if ( (string_a == NULL) && (string_b == NULL)) return  0;
	if ( (string_a == NULL) && (string_b != NULL)) return -1;

	cmp_result = g_utf8_collate (string_a, string_b);

	g_free (string_a);
	g_free (string_b);

	return cmp_result;
}


static void
_print_listnames (BmpPlaylistItem *item, gpointer data)
{
	g_message ("LIST: %s", item->listname);
}

static void
playlists_sort_func (BmpPlaylist *self, gconstpointer data2)
{
	self->private->playlists->items =
			g_list_sort (self->private->playlists->items,
			(GCompareFunc)playlists_compare_func);

}

static void
tracklist_sort_func (BmpListType *list, gconstpointer data2)
{
	gpointer item;

	item = g_list_nth (list->items, list->current);
	list->items = g_list_sort_with_data (list->items, (GCompareDataFunc)tracklist_compare_func, (gpointer)data2);
	list->current = g_list_position (list->items, item);
}

static void
_playlists_clear_current (gpointer item, BmpPlaylist *self)
{
	BmpPlaylistItem *list_item = (BmpPlaylistItem*)item;
	BmpListType *list;

	list = bmp_playlist_tracklist_get (self, list_item->listname);

	list->current = -1;
}

void
bmp_playlist_sort_current_list (BmpPlaylist *self)
{
	list_manager_list_sort (self->private->list_manager,
		bmp_playlist_get_nth_tracklist_name(self, g_object_get_int(bmp_system_control, "current_tracklist")),
		NULL);

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_LIST_SORTED],
                   0, NULL);

}

void
bmp_playlist_sort_current_list_by_tag (BmpPlaylist *self, BmpTagId tag_id)
{
	list_manager_list_sort (self->private->list_manager,
		bmp_playlist_get_nth_tracklist_name(self, g_object_get_int(bmp_system_control, "current_tracklist")),
		GINT_TO_POINTER(tag_id));

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_LIST_SORTED],
                   0, NULL);
}


void
bmp_playlist_clear_all_current (BmpPlaylist *self)
{
	g_return_if_fail (BMP_IS_PLAYLIST (self));

	g_list_foreach (self->private->playlists->items, (GFunc)_playlists_clear_current, self);
}

static void
load_playlist (BmpPlaylist *self, const gchar *filename)
{
	BmpListType *list;
	GList *uri_list;
	GError *error = NULL;
	gchar *name, *aux;
	gchar *buffer;
	gchar **filenames;
	gint n = 0;

	g_return_if_fail (BMP_IS_PLAYLIST (self));

	aux =
	    g_path_get_basename (filename);
	name =
	    g_strndup (aux, strlen(aux) - 4);
	g_free (aux);
	list =
	    bmp_playlist_tracklist_new (self, name);

	/* FIMXE: Use GURI for that */
	aux =
	    g_strconcat ("file://", filename, NULL);
	uri_list =
	    get_uris__m3u (aux);
	g_free (aux);

	while (uri_list) {
		bmp_playlist_insert_tracklist_item (self, name, uri_list->data);
		uri_list = g_list_next (uri_list);
	}

	g_list_foreach (uri_list, (GFunc) g_free, NULL);
	g_list_free (uri_list);

	if (!strcmp(name, self->private->pre_current_name) && (self->private->pre_current_position != -1)) {

		list->current = self->private->pre_current_position;
		self->private->pre_current_position = -1;
	}
	else
		list->current = -1;

	g_free(name);
}

static void
playlist_removed_func (const gchar *filename, BmpPlaylist *self)
{
}

static void
playlist_added_func (const gchar *filename, BmpPlaylist *self)
{
	gchar *fqname;

	fqname = g_build_filename (bmp_configuration_key_fetch (bmp_configuration, "bmp", "playlists-path"),
				   filename, NULL);
	load_playlist (self, fqname);
	g_free (fqname);

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_LIST_ADDED],
                   0, NULL);
}

static void
bmp_playlist_init (BmpPlaylist  *self)
{
	gchar *listname;
	gint _idx;

        self->private = g_new0(BmpPlaylistPrivate, 1);

        self->private->dispose_has_run = FALSE;

	self->private->pre_current_position = -1;
	self->private->pre_current_name = NULL;

	self->private->list_manager = list_manager_new();

	self->private->playlists = g_new0(BmpListType,1);
	self->private->playlists->name = NULL;
	self->private->playlists->column_data_func = (ColumnDataFunc)playlists_column_data_func;
	self->private->playlists->row_get_selected_func = (RowGetSelectedFunc)playlists_get_selected_func;
	self->private->playlists->row_set_selected_func = (RowSetSelectedFunc)playlists_set_selected_func;
	self->private->playlists->list_sort_func = (ListSortFunc)playlists_sort_func;
	self->private->playlists->on_display_func = NULL;
	self->private->playlists->get_dirty_func = (GetDirtyFunc)playlists_get_dirty_func;
	self->private->playlists->set_dirty_func = (SetDirtyFunc)playlists_set_dirty_func;
	self->private->playlists->item_destroy_func = (GDestroyNotify)playlists_item_destroy_func;
	self->private->playlists->set_rendered_func = (SetRenderedFunc)playlists_set_rendered_func;
	self->private->playlists->get_rendered_func = (GetRenderedFunc)playlists_get_rendered_func;
	self->private->playlists->items = NULL;
	self->private->playlists->current = -1;
	self->private->playlists->last_selected = -1;
	self->private->playlists->multiple_selected = FALSE;

#if 0
	fam_monitor_watch_new (bmp_fam_monitor, "playlists",
				bmp_configuration_key_fetch(bmp_configuration, "bmp", "playlists-path"),
				playlist_added_func,
				playlist_removed_func,
				self,
				NULL);
#endif

    listname =
	bmp_configuration_key_fetch(bmp_configuration,"bmp","current-playlist");
    _idx =
	bmp_configuration_key_fetch_int(bmp_configuration,"bmp","current-playlist-position");

    if (! listname) listname = g_strdup("");

    bmp_playlist_set_pre_current_name (self, listname);
    bmp_playlist_set_pre_current_position (self, _idx);

    g_free (listname);

    g_message("%s:\t\tINITIALIZED", __FILE__);
}

static void
bmp_playlist_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static void
bmp_playlist_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static GObject *
bmp_playlist_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                /* Invoke parent constructor. */
                BmpPlaylistClass *klass;
                GObjectClass *parent_class;
                klass = BMP_PLAYLIST_CLASS (g_type_class_peek (BMP_TYPE_PLAYLIST));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }

        /* do stuff. */

        return obj;
}

static void
bmp_playlist_dispose (GObject *gobject)
{
    BmpPlaylist *self = (BmpPlaylist *) gobject;

    if (self->private->dispose_has_run)
        return;

    self->private->dispose_has_run = TRUE;

    bmp_playlist_save_lists (self);

    g_message("%s:\t\tSTOPPED", __FILE__);
}

static void
bmp_playlist_finalize (GObject *gobject)
{
    BmpPlaylist *self = (BmpPlaylist *) gobject;

    /*
     * Here, complete object destruction.
     * You might not need to do much...
     */
    /*g_free (self->private);*/
}

BmpPlaylist*
bmp_playlist_new (void)
{
  BmpPlaylist *list;
  list = g_object_new (bmp_playlist_get_type (), NULL);

  return list;
}

static void
bmp_playlist_class_init (BmpPlaylistClass *g_class)

{
    GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);

    gobject_class->set_property = bmp_playlist_set_property;
    gobject_class->get_property = bmp_playlist_get_property;
    gobject_class->dispose = bmp_playlist_dispose;
    gobject_class->finalize = bmp_playlist_finalize;
    gobject_class->constructor = bmp_playlist_constructor;

    signals[BMP_PLAYLIST_SIGNAL_LIST_DELETED] =
        g_signal_new ("list-deleted",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, list_deleted),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);

    signals[BMP_PLAYLIST_SIGNAL_LIST_ADDED] =
        g_signal_new ("list-added",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, list_added),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);

    signals[BMP_PLAYLIST_SIGNAL_LIST_ADDED] =
        g_signal_new ("list-renamed",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, list_renamed),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);

    signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_LIST_SORTED] =
        g_signal_new ("tracklist-list-sorted",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, tracklist_list_sorted),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);

    signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_REMOVED] =
        g_signal_new ("tracklist-items-removed",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, tracklist_items_removed),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);
    signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_ADDED] =
        g_signal_new ("tracklist-items-added",
                      G_OBJECT_CLASS_TYPE (gobject_class),
                      G_SIGNAL_RUN_FIRST,
                      G_STRUCT_OFFSET (BmpPlaylistClass, tracklist_items_added),
                      NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE,
                      0);
}

BmpListType*
bmp_playlist_get_playlists (BmpPlaylist *self)
{
	return self->private->playlists;
}

const gchar*
bmp_playlist_get_nth_tracklist_name (BmpPlaylist *self, gint _idx)
{
	BmpPlaylistItem *item;

	g_return_val_if_fail (BMP_IS_PLAYLIST (self), NULL);

	if (! self->private->playlists->items)
	    return NULL;
	if (g_list_length (self->private->playlists->items) == 0)
	    return NULL;
	if (_idx > (g_list_length (self->private->playlists->items)-1))
	    return NULL;

	item = LIST_GET_NTH_ITEM(self->private->playlists, _idx);

	return item->listname;
}

void
bmp_playlist_set_pre_current_position (BmpPlaylist *self, gint _idx)
{
	g_return_if_fail (BMP_IS_PLAYLIST (self));

	self->private->pre_current_position = _idx;
}

void
bmp_playlist_set_pre_current_name (BmpPlaylist *self, const gchar *name)
{
	g_return_if_fail (BMP_IS_PLAYLIST (self));

	self->private->pre_current_name = g_strdup (name);
}

const gchar*
bmp_playlist_get_current_tracklist_name (BmpPlaylist *self)
{
	BmpPlaylistItem *item;
	gint _idx;

	g_return_val_if_fail (BMP_IS_PLAYLIST (self), NULL);

	if (! self->private->playlists)
	    return NULL;
	if (! self->private->playlists->items)
	    return NULL;
	if (g_list_length(self->private->playlists->items) == 0)
	    return NULL;

	_idx = g_object_get_int (bmp_system_control, "current_tracklist");

	if (_idx < 0)
	    return NULL;

	item = LIST_GET_NTH_ITEM(self->private->playlists, _idx);

	return item->listname;
}

gint
bmp_playlist_get_tracklist_by_name (BmpPlaylist *self, const gchar *name)
{
	BmpPlaylistItem *item;
	GList *iter_list;
	gint ctr = 0;

	g_return_val_if_fail (BMP_IS_PLAYLIST (self), -1);

	iter_list = self->private->playlists->items;

	while (iter_list)
	{
		item = iter_list->data;
		if (!strcmp(name, item->listname)) break;
		ctr++;

		iter_list = g_list_next (iter_list);
	}

	return ctr;
}

void
bmp_playlist_tracklist_shuffle (BmpPlaylist *self, const gchar *name)
{
	g_return_if_fail (BMP_IS_PLAYLIST (self));

	if (! name) return;

	list_manager_list_shuffle (self->private->list_manager, name);
}

static gboolean
playlist_rename (BmpPlaylist *self, const gchar *listname, const gchar *newname)
{
	BmpListType *tracklist;
	BmpPlaylistItem *playlist_item;
	GList *iter_list;
	gchar *filename, *aux1;

	g_return_val_if_fail (BMP_IS_PLAYLIST (self), FALSE);

	tracklist = list_manager_list_get (self->private->list_manager, newname);

	if (tracklist) return FALSE;

	iter_list = g_list_first(self->private->playlists->items);

	while (iter_list)
	{
		playlist_item = iter_list->data;

		if (!strcmp(playlist_item->listname, listname))
		{
			aux1 = g_strconcat (newname, ".m3u", NULL);
			filename = g_build_filename (bmp_paths[BMP_PATH_PLAYLISTS], aux1, NULL);

			g_rename (playlist_item->filename, filename);

			g_free (aux1);
			g_free (filename);
			break;
		}
		iter_list = g_list_next (iter_list);
	}

	list_manager_list_rename (self->private->list_manager, listname, newname);

	return TRUE;
}

gboolean
bmp_playlist_rename_playlist (BmpPlaylist *self, gint _idx, const gchar *name)
{
	BmpPlaylistItem *item;
	GList *llink_current = NULL, *llink_active = NULL, *llink = NULL;
	gchar *name_old, *aux1, *aux2;

	llink = g_list_nth (self->private->playlists->items, self->private->playlists->current);
	llink_current = LIST_GET_NTH_ITEM_GLIST(self->private->playlists, g_object_get_int (bmp_system_control, "current_tracklist"));
	llink_active  = LIST_GET_NTH_ITEM_GLIST(self->private->playlists, g_object_get_int (bmp_system_control, "active_tracklist"));

	item = LIST_GET_NTH_ITEM(self->private->playlists, _idx);
	name_old = g_strdup (item->listname);
	if (! playlist_rename (self, name_old, name)) return FALSE;

	aux1 = g_strconcat(name, ".m3u", NULL);
	aux2 = g_build_filename (bmp_paths[BMP_PATH_PLAYLISTS], aux1, NULL);
	g_free (item->listname);
	item->listname = g_strdup(name);
	g_free (item->filename);
	item->filename = g_strdup(aux2);

	self->private->playlists->list_sort_func (self, NULL);

	self->private->playlists->current = g_list_position(self->private->playlists->items, llink);
	g_object_set (bmp_system_control, "current_tracklist", (gint)g_list_position(self->private->playlists->items, (GList*)llink_current), NULL);
	g_object_set (bmp_system_control, "active_tracklist",  (gint)g_list_position(self->private->playlists->items, (GList*)llink_active), NULL);

	g_free (aux1);
	g_free (aux2);
	g_free (name_old);

	return TRUE;
}

void
bmp_playlist_remove_all_playlists (BmpPlaylist *self)
{
	BmpPlaylistItem *item;
	GList *iter_list;
	gchar *name;

	iter_list = self->private->playlists->items;

	while (iter_list)
	{
		item = iter_list->data;
		g_unlink (item->filename);

		name = g_strdup (item->listname);
		list_manager_list_remove (self->private->list_manager, name);

		iter_list = g_list_next (iter_list);
	}

	self->private->playlists->current = -1;
	self->private->playlists->items = NULL;

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_LIST_DELETED],
                   0);
}

void
bmp_playlist_remove_playlist (BmpPlaylist *self, gint _idx)
{
	BmpPlaylistItem *item;
	GList *llink_current = NULL, *llink_active = NULL, *llink = NULL;
	gchar *name;

	item = LIST_GET_NTH_ITEM(self->private->playlists, _idx);
	g_unlink (item->filename);

	llink_current = LIST_GET_NTH_ITEM_GLIST(self->private->playlists, g_object_get_int (bmp_system_control, "current_tracklist"));
	llink_active  = LIST_GET_NTH_ITEM_GLIST(self->private->playlists, g_object_get_int (bmp_system_control, "active_tracklist"));

	llink = g_list_nth (self->private->playlists->items, self->private->playlists->current);

	name = g_strdup (item->listname);
	LIST_REMOVE_ITEM(self->private->playlists, _idx);
	list_manager_list_remove (self->private->list_manager, name);

	self->private->playlists->current = g_list_position (self->private->playlists->items, llink);

	g_object_set (bmp_system_control, "current_tracklist", (gint)g_list_position(self->private->playlists->items, (GList*)llink_current), NULL);
	g_object_set (bmp_system_control, "active_tracklist",  (gint)g_list_position(self->private->playlists->items, (GList*)llink_active), NULL);

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_LIST_DELETED],
                   0);
}

static void
playlist_remove_playlist_by_name (BmpPlaylist *self, const gchar *name)
{
}

static void
insert_playlist_item (BmpPlaylist *self, const gchar *name)
{
	BmpPlaylistItem *playlist_item = NULL;
	GList *llink;
	gchar *basename;
	gchar *filename;

	llink = g_list_nth (self->private->playlists->items, self->private->playlists->current);

	basename = g_strconcat (name, ".m3u", NULL);
	filename = g_build_filename (bmp_paths[BMP_PATH_PLAYLISTS], basename, NULL);
	g_free (basename);

	playlist_item = g_new0(BmpPlaylistItem,1);
	playlist_item->filename = filename;
	playlist_item->listname = g_strdup(name);
	playlist_item->selected = FALSE;
	playlist_item->dirty = FALSE;
	playlist_item->rendered = FALSE;
	LIST_ADD_ITEM(self->private->playlists, playlist_item);

	self->private->playlists->list_sort_func (self, NULL);
	self->private->playlists->current = g_list_position (self->private->playlists->items, llink);
}

static BmpFileItem*
file_item_new (const gchar *uri) {

    BmpFileItem *file_item = NULL;

    g_return_val_if_fail (uri != NULL, NULL);

    file_item = g_new0(BmpFileItem,1);

    file_item->uri = g_strdup(uri);
    file_item->metadata = NULL;
    file_item->metadata_attempted = FALSE;
    file_item->selected = FALSE;
    file_item->dirty = FALSE;
    file_item->rendered = FALSE;

    return file_item;
}

void
bmp_playlist_insert_tracklist_item (BmpPlaylist *self, const gchar *listname, const gchar *uri)
{
	BmpFileItem *file_item = NULL;
        GList *items = NULL;
        GError *error = NULL;
	GURI *g_uri;

	g_uri =
	    gnet_uri_new (uri);

	if (! g_uri)
	    return;

	gnet_uri_unescape (g_uri);

	/* FIXME: Quite liberal */
	if (!G_URI_IS_LOCAL(uri))
	    goto add_item;

        if (g_file_test(g_uri->path, G_FILE_TEST_IS_DIR)) {

               dir_foreach(g_uri->path, (DirForeachFunc)collect_path,
                           &items, &error);

               while (items) {

                       gchar *new_uri =
			    g_filename_to_uri (items->data, NULL, NULL);

                       bmp_playlist_insert_tracklist_item (self, listname, new_uri);

                       items =
			    g_list_next (items);
               }

               g_list_foreach (g_list_first(items), (GFunc)g_free, NULL);

        } else {

	    if (! uri)
		return;

	    if (! g_uri->path) {
		gnet_uri_delete (g_uri);
		return;
	    }

	    if (! g_file_test (g_uri->path, G_FILE_TEST_EXISTS)) {
		gnet_uri_delete (g_uri);
		return;
	    }

	    add_item:

	    /* Being here it could be as well that it's a container file in itself, like .m3u or .pls */
	    if (str_has_suffix_nocase(uri, ".pls") || str_has_suffix_nocase(uri, ".m3u")) {

		GList *uri_list = NULL;

		uri_list =
		    get_uris__playlist (gnet_uri_get_string(g_uri));

		while (uri_list) {
		    file_item =
			file_item_new (uri_list->data);
		    LIST_ADD_ITEM(bmp_playlist_tracklist_get (self, listname), file_item);
		    uri_list =
			g_list_next (uri_list);
		}

		g_list_foreach (uri_list, (GFunc) g_free, NULL);
		g_list_free (uri_list);

	    } else {
		file_item =
		    file_item_new (gnet_uri_get_string(g_uri));
		LIST_ADD_ITEM(bmp_playlist_tracklist_get (self, listname), file_item);
	    }

        }

    gnet_uri_delete (g_uri);
}

int
bmp_playlist_insert_tracklist_item_position (BmpPlaylist *self, const gchar *listname, const gchar *uri, gint _idx)
{
	BmpListType *list;
	BmpFileItem *file_item = NULL;
	GList *items = NULL;
	GError *error = NULL;
	GURI *g_uri;

	list =
	    bmp_playlist_tracklist_get (self, listname);

	g_uri =
	    gnet_uri_new (uri);

	if (! g_uri)
	    return -1;

	gnet_uri_unescape (g_uri);

	if (str_has_suffix_nocase(uri, ".pls") || str_has_suffix_nocase(uri, ".m3u")) {

		GList *uri_list = NULL;

		uri_list =
		    get_uris__playlist (gnet_uri_get_string(g_uri));

		while (uri_list) {
		    file_item =
			file_item_new (uri_list->data);
		    LIST_ADD_ITEM_POSITION(bmp_playlist_tracklist_get (self, listname), file_item, _idx++);
		    uri_list =
			g_list_next (uri_list);
		}

		g_list_foreach (uri_list, (GFunc) g_free, NULL);
		g_list_free (uri_list);

	} else if (g_file_test(g_uri->path, G_FILE_TEST_IS_REGULAR)) {
			file_item = file_item_new (gnet_uri_get_string(g_uri));
			LIST_ADD_ITEM_POSITION(list, file_item, _idx++);
	} else if (g_file_test(g_uri->path, G_FILE_TEST_IS_DIR)) {

		dir_foreach(g_uri->path, (DirForeachFunc)collect_path,
                 	    &items, &error);

		while (items) {
			gchar *new_uri =
			    g_filename_to_uri (items->data, NULL, NULL);

			/* Do not need to increase _idx here, as if its a file,
			 * above code will handle that ... */
			_idx = bmp_playlist_insert_tracklist_item_position (self, listname, new_uri, _idx);
			items = g_list_next (items);
		}

		g_list_foreach (g_list_first(items), (GFunc)g_free, NULL);
        }

    if (g_object_get_boolean(bmp_system_control,"shuffle"))
	    bmp_playlist_tracklist_shuffle (self, listname);

    gnet_uri_delete (g_uri);

    return _idx;
}

void
bmp_playlist_tracklist_remove_all (BmpPlaylist *self, const gchar *listname)
{
	BmpListType *list;

	list = bmp_playlist_tracklist_get (self, listname);

	list->items = NULL;
	list->current = 0;
	list->current_linear = 0;
	list->last_selected = LAST_SELECTED_NONE;
	list->adjustment = 0.0;

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_REMOVED],
                   0, NULL);
	/* FIXME: Free old items */
}

void
bmp_playlist_tracklist_remove_selected (BmpPlaylist *self, const gchar *listname)
{
	GList *new_items;
	BmpListType *list;
	BmpFileItem *file_item;
	gint n, new_current, new_ctr;

	list = bmp_playlist_tracklist_get (self, listname);
	new_items = NULL;
	new_ctr = 0;
	new_current = -1;

	for (n = 0; n < g_list_length(list->items); n++)
	{
		file_item = LIST_GET_NTH_ITEM (list, n);

		/* FIXME: is it really ok to access privately? by doing this be basically say, the function pointers
			  in the list struct are only for the chromalist widget.. */
		if (!file_item->selected)
		{
			new_items = g_list_append (new_items, file_item);
			if (n == list->current)
				new_current = new_ctr;
			new_ctr++;
		}
	}
	list->items = new_items;
	list->current = new_current;

	if ((list->last_selected > g_list_length(list->items)-1) && new_items)
	    list->last_selected = g_list_length(list->items)-1;
	else if (! new_items)
	    list->last_selected = LAST_SELECTED_NONE;

	if (list->last_selected != LAST_SELECTED_NONE)
	{
	    file_item = g_list_nth(list->items, list->last_selected)->data;
	    list->row_set_selected_func ((gpointer)file_item, TRUE);
	}
	else
	{
	    list->last_selected = LAST_SELECTED_NONE;
	}

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_REMOVED],
                   0, NULL);
	/* FIXME: Free old items */
}

void
bmp_playlist_tracklist_remove_unselected (BmpPlaylist *self, const gchar *listname)
{
	GList *new_items;
	BmpListType *list;
	BmpFileItem *file_item;
	gint n, new_ctr, new_current;

	list = bmp_playlist_tracklist_get (self, listname);
	new_items = NULL;
	new_ctr = 0;
	new_current = -1;

	for (n = 0; n < g_list_length(list->items); n++)
	{
		file_item = LIST_GET_NTH_ITEM (list, n);

		/* fixme: is it really ok to access privately? by doing this be basically say, the function pointers
			  in the list struct are only for the chromalist widget.. */
		if (file_item->selected)
		{
			new_items = g_list_append (new_items, file_item);
			if (n == list->current)
				new_current = new_ctr;
			new_ctr++;
		}
	}
	list->items = new_items;
	list->current = new_current;
	list->last_selected = 0;

        g_signal_emit (G_OBJECT (self),
                   signals[BMP_PLAYLIST_SIGNAL_TRACKLIST_ITEMS_REMOVED],
                   0, NULL);
	/* FIXME: Free old items */
}


BmpListType*
bmp_playlist_tracklist_get (BmpPlaylist *self, const gchar *name)
{
	return name?list_manager_list_get (self->private->list_manager, name):NULL;
}

BmpListType*
bmp_playlist_tracklist_new (BmpPlaylist *self, const gchar *name)
{
	BmpListType *list;

	list = list_manager_list_get (self->private->list_manager,
				      name);

	if (list) return NULL;

	list = list_manager_list_new (self->private->list_manager,
				      name,
				      (ColumnDataFunc)tracklist_column_data_func,
				      (RowGetSelectedFunc)tracklist_get_selected_func,
				      (RowSetSelectedFunc)tracklist_set_selected_func,
				      (OnDisplayFunc)tracklist_on_display_func,
				      (ItemSwapFunc)tracklist_item_swap_func,
				      (ListSortFunc)tracklist_sort_func,
				      (SetDirtyFunc)tracklist_set_dirty_func,
				      (GetDirtyFunc)tracklist_get_dirty_func,
				      (SetRenderedFunc)tracklist_set_rendered_func,
				      (GetRenderedFunc)tracklist_get_rendered_func,
				      (GDestroyNotify)tracklist_item_destroy_func);

	insert_playlist_item (self, name);

	if (g_object_get_boolean (bmp_system_control, "shuffle")) {
	    bmp_playlist_tracklist_shuffle (self, name);
	}

	return list;
}

static void
save_playlist (BmpPlaylistItem *playlist_item, BmpPlaylist *self)
{
	BmpListType *list;
	BmpFileItem *item;
	GList *iter_list;
	GError *error = NULL;
	gchar *buffer = NULL, *aux;

	list = bmp_playlist_tracklist_get (self, playlist_item->listname);
	iter_list = list->items;

	while (iter_list)
	{
		item = iter_list->data;
		if (buffer!=NULL)
		{
			aux = g_strconcat (buffer, "\n", item->uri, NULL);
			g_free (buffer);
			buffer = g_strdup(aux);
			g_free (aux);
		}
		else
		{
			buffer = g_strconcat (item->uri, NULL);
		}

		iter_list = g_list_next(iter_list);
	}

	if ( (! buffer) || (! strlen(buffer))) {
		    buffer = g_strdup("\n");
	}

	g_file_set_contents (playlist_item->filename, buffer, -1, &error);

	g_free (buffer);

	if (error) {
		g_message("SAVE playlist [%s] ERROR: %s", playlist_item->filename, error->message);
	}
}

void
bmp_playlist_save_lists (BmpPlaylist *self)
{
	BmpListType *list;
	const gchar *name;

	g_return_if_fail (BMP_IS_PLAYLIST (self));

	if (!bmp_configuration)
	    return;

	if (g_list_length(self->private->playlists->items) == 0) return;
	g_list_foreach (self->private->playlists->items, (GFunc) save_playlist, self);

	/* FIXME: This doesn't belong here, it's an UI issue */
	name = bmp_playlist_get_current_tracklist_name (self);
	bmp_configuration_key_set_string (bmp_configuration,
		"bmp",
		"current-playlist",
		name);

	list =
	    bmp_playlist_tracklist_get (self, name);
	if (list)
	    bmp_configuration_key_set_int (bmp_configuration,
		"bmp",
		"current-playlist-position",
		list->current);
}

void
bmp_playlist_load_playlists (BmpPlaylist *self)
{
	BmpListType *list;
	GDir *dir;
	GError *error = NULL;
	gchar *fqname, *listname;
	const gchar* filename;
	gint current_list = 0;

	g_return_if_fail (BMP_IS_PLAYLIST (self));

	dir = g_dir_open (bmp_configuration_key_fetch (bmp_configuration, "bmp", "playlists-path"), 0, &error);

	if (error)
	{
		g_message("LOAD PLAYLISTS ERORR: %s", error->message);
		return;
	}

	while (42)
	{
		filename = g_dir_read_name (dir);
		if (filename == NULL) break;
		if (str_has_suffix_nocase(filename, "m3u"))
		{
			fqname = g_build_filename (bmp_paths[BMP_PATH_PLAYLISTS], filename, NULL);
			load_playlist (self, fqname);
			g_free (fqname);
			current_list++;
		}
	}
	g_dir_close (dir);

	if (! current_list) {
		listname = g_strdup ("Default");
		list = bmp_playlist_tracklist_new (self, listname);
		if (list) {
			bmp_playlist_set_pre_current_name (self, listname);
			bmp_playlist_set_pre_current_position (self, -1);
			bmp_configuration_key_set_string (bmp_configuration, "bmp", "current-playlist", "Default");
		} else
			g_free (listname);
	}

	self->private->playlists->list_sort_func (self, NULL);
}

gint
bmp_playlist_get_current_track_idx (BmpPlaylist *self)
{
	BmpListType *list;
	const gchar *name;

	name = bmp_playlist_get_nth_tracklist_name (self, g_object_get_int (bmp_system_control, "active_tracklist"));
	list = bmp_playlist_tracklist_get (self, name);

	if (!list) return 0;
	if (!list->items) return 0;
	if (list->current < 0) return 0;

	return list->current;
}

gchar*
bmp_playlist_get_current_uri (BmpPlaylist *self)
{
	BmpListType *list;
	BmpFileItem *item;
	const gchar *uri;
        const gchar *name;

	name = bmp_playlist_get_nth_tracklist_name (self, g_object_get_int (bmp_system_control, "active_tracklist"));
	list = bmp_playlist_tracklist_get (self, name);

	if (! list) return NULL;
	if (! list->items) return NULL;
	if (list->current < 0) return NULL;

	item = g_list_nth(((BmpListType*)list)->items, (gint)(BmpListType*)list->current)->data;
	if (! item) return NULL;

	tracklist_on_display_func (item);

	uri = g_hash_table_lookup (item->metadata, bmp_metadata_get_id_static(BMP_DATUM_LOCATION));

	return uri;
}

gchar*
bmp_playlist_get_current_title (BmpPlaylist *self)
{
	BmpListType *list;
	BmpFileItem *item;
	const gchar *title;
        const gchar *name;

	name = bmp_playlist_get_nth_tracklist_name (self, g_object_get_int (bmp_system_control, "active_tracklist"));
	list = bmp_playlist_tracklist_get (self, name);

	if (! list) return NULL;
	if (! list->items) return NULL;
	if (list->current < 0) return NULL;

	item = g_list_nth(((BmpListType*)list)->items, (gint)(BmpListType*)list->current)->data;
	if (! item) return NULL;


	tracklist_on_display_func (item);

	title = bmp_metadata_get_titlestring (item->metadata);
#if 0
	title = g_strdup_printf("%d. %s", list->current+1, aux);
	g_free (aux);
#endif

	return title;
}
