#ifndef BMP_PLAYLIST_H
#define BMP_PLAYLIST_H

#include <glib-object.h>
#include <list_manager.h>
#include <bmp_metadata.h>

#define BMP_TYPE_PLAYLIST		  (bmp_playlist_get_type ())
#define BMP_PLAYLIST(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_PLAYLIST, BmpPlaylist))
#define BMP_PLAYLIST_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_PLAYLIST, BmpPlaylistClass))
#define BMP_IS_PLAYLIST(obj)	  (GTK_CHECK_TYPE ((obj), BMP_TYPE_PLAYLIST))
#define BMP_IS_PLAYLIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_PLAYLIST))
#define BMP_PLAYLIST_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_PLAYLIST, BmpPlaylistClass))

typedef struct _BmpPlaylist BmpPlaylist;
typedef struct _BmpPlaylistClass BmpPlaylistClass;
typedef struct _BmpPlaylistPrivate BmpPlaylistPrivate;

struct _BmpPlaylist {
	GObject parent;

        BmpPlaylistPrivate *private;
};

struct _BmpPlaylistClass {
	GObjectClass parent;

	void (*list_deleted) (BmpPlaylist *self);
	void (*list_added)   (BmpPlaylist *self);
	void (*list_renamed) (BmpPlaylist *self);

	void (*tracklist_list_sorted)    (BmpPlaylist *self);
	void (*tracklist_items_removed)  (BmpPlaylist *self);
	void (*tracklist_items_added)    (BmpPlaylist *self);
};

/* used by LIST_TYPE_MANAGER */
GType bmp_playlist_get_type (void);

typedef struct {
	gchar *uri;
	GHashTable *metadata;
	gboolean metadata_attempted;

	gboolean selected;
	gboolean dirty;
	gboolean rendered;

	gboolean locked;

} BmpFileItem;
#define BMP_FILE_ITEM(item) (BmpFileItem*)(item)

typedef struct {
	gchar *filename;
	gchar *listname;

	gboolean selected;
	gboolean dirty;
	gboolean rendered;

} BmpPlaylistItem;
#define BMP_PLAYLIST_ITEM(item) (BmpPlaylistItem*)(item)

BmpPlaylist*
bmp_playlist_new (void);

void
bmp_playlist_setup (BmpPlaylist *self);

BmpListType*
bmp_playlist_get_playlists(BmpPlaylist *self);

gint
bmp_playlist_get_tracklist_by_name (BmpPlaylist *self, const gchar *name);

void
bmp_playlist_set_pre_current_name (BmpPlaylist *self, const gchar *name);

void
bmp_playlist_set_pre_current_position (BmpPlaylist *self, gint _idx); 

const gchar* 
bmp_playlist_get_current_tracklist_name (BmpPlaylist *self);

const gchar*
bmp_playlist_get_nth_tracklist_name (BmpPlaylist *self, gint _idx);

void
bmp_playlist_tracklist_shuffle (BmpPlaylist *self, const gchar *name);
	
gchar*
bmp_playlist_get_current_title (BmpPlaylist *self);
	
gint
bmp_playlist_get_current_track_idx (BmpPlaylist *self);
	
gchar*
bmp_playlist_get_current_uri (BmpPlaylist *self);
	
gboolean
bmp_playlist_rename_playlist (BmpPlaylist *self, gint _idx, const gchar *name);

void
bmp_playlist_remove_playlist (BmpPlaylist *self, gint _idx);

void
bmp_playlist_remove_all_playlists (BmpPlaylist *self);
	
BmpListType*
bmp_playlist_tracklist_new (BmpPlaylist *self, const gchar *name);

BmpListType*
bmp_playlist_tracklist_get (BmpPlaylist *self, const gchar *name);

#define BMP_PLAYLIST_GET_CURRENT(list) \
	bmp_playlist_tracklist_get (list, bmp_playlist_get_current_tracklist_name(list))	

void
bmp_playlist_insert_tracklist_item (BmpPlaylist *self, const gchar *listname, const gchar *uri);

int
bmp_playlist_insert_tracklist_item_position (BmpPlaylist *self, const gchar *listname, const gchar *uri, gint _idx);

#define BMP_TRACKLIST_GET_CURRENT(tracklist) \
	g_list_nth(((BmpListType*)tracklist)->items, (BmpListType*)tracklist->current)->data
	
void
bmp_playlist_tracklist_remove_selected (BmpPlaylist *self, const gchar *listname);

void
bmp_playlist_tracklist_remove_unselected (BmpPlaylist *self, const gchar *listname);

void
bmp_playlist_tracklist_remove_all (BmpPlaylist *self, const gchar *listname);

void
bmp_playlist_save_lists (BmpPlaylist *self);

void
bmp_playlist_load_playlists (BmpPlaylist *self);

void
bmp_playlist_rename (BmpPlaylist *self, const gchar *listname, const gchar *newname);

void
bmp_playlist_clear_all_current (BmpPlaylist *self);

void
bmp_playlist_sort_current_list (BmpPlaylist *self);
	
void
bmp_playlist_sort_current_list_by_tag (BmpPlaylist *self, BmpTagId tag_id);
			
#endif
