/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#ifndef BMP_SYSTEM_CONTROL_H
#define BMP_SYSTEM_CONTROL_H

/*
 * Potentially, include other headers on which this header depends.
 */
#include <glib-object.h>

#define BMP_TYPE_SYSTEM_CONTROL		  (bmp_system_control_get_type ())
#define BMP_SYSTEM_CONTROL(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), BMP_TYPE_SYSTEM_CONTROL, BmpSystemControl))
#define BMP_SYSTEM_CONTROL_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), BMP_TYPE_SYSTEM_CONTROL, BmpSystemControlClass))
#define BMP_IS_SYSTEM_CONTROL(obj)	  (GTK_CHECK_TYPE ((obj), BMP_TYPE_SYSTEM_CONTROL))
#define BMP_IS_SYSTEM_CONTROL_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), BMP_TYPE_SYSTEM_CONTROL))
#define BMP_SYSTEM_CONTROL_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), BMP_TYPE_SYSTEM_CONTROL, BmpSystemControlClass))

typedef struct _BmpSystemControl BmpSystemControl;
typedef struct _BmpSystemControlClass BmpSystemControlClass;
typedef struct _BmpSystemControlPrivate BmpSystemControlPrivate;

struct _BmpSystemControl {
	GObject parent;

        BmpSystemControlPrivate *private;
};

struct _BmpSystemControlClass {
	GObjectClass parent;

    	void (*track_change)		(BmpSystemControl * self);

    	void (*set_current_tracklist)	(BmpSystemControl * self, gint list);
    	void (*set_stream_pos)		(BmpSystemControl * self, gint stream_pos);
    	void (*set_playstatus)	        (BmpSystemControl * self, gint playstatus);
    	void (*set_volume)	        (BmpSystemControl * self, gint volume);
};

typedef enum {
	BMP_SYSTEM_MESSAGE_CURRENT,
	BMP_SYSTEM_MESSAGE_NEXT,
	BMP_SYSTEM_MESSAGE_PREV,
	BMP_SYSTEM_MESSAGE_STOP,
	BMP_SYSTEM_MESSAGE_PLAY,
	BMP_SYSTEM_MESSAGE_PAUSE,
	BMP_SYSTEM_MESSAGE_SEEK,
	BMP_SYSTEM_MESSAGE_PLAY_LIST,
	BMP_SYSTEM_MESSAGE_PLAY_TRACK,
	BMP_SYSTEM_MESSAGE_PLAY_LIST_TRACK,

	BMP_SYSTEM_MESSAGE_SET_CURRENT_TRACKLIST,

	BMP_SYSTEM_MESSAGE_VOLUME,
	BMP_SYSTEM_MESSAGE_EQ,

	BMP_SYSTEM_MESSAGE_BITRATE,
	BMP_SYSTEM_MESSAGE_SAMPLERATE,

	BMP_SYSTEM_MESSAGE_REPEAT,
	BMP_SYSTEM_MESSAGE_SHUFFLE,

	BMP_SYSTEM_MESSAGE_UI_START,
	BMP_SYSTEM_MESSAGE_UI_STOP,

	BMP_SYSTEM_MESSAGE_SHOW_PL_WINDOW,

	BMP_SYSTEM_MESSAGE_SAVE_STATE,

	BMP_SYSTEM_MESSAGE_QUIT,

	BMP_N_SYSTEM_MESSAGES
} BmpSystemMessage;

typedef struct {
	BmpSystemMessage message_type;	
	gpointer         user_data;
	GDestroyNotify   user_data_destroy;
} bmp_message_t;

#define BMP_DBUS_OBJECT_PATH "/org/beepmediaplayer/SystemControl/BmpSystemControl"
#define BMP_DBUS_NAME        "org.beepmediaplayer.SystemControl"

/* API */
GType bmp_system_control_get_type (void);

BmpSystemControl*
bmp_system_control_new (gboolean no_remote);

void
bmp_system_message_send (BmpSystemControl *self, 
		 	 BmpSystemMessage message_type,
			 gpointer user_data,
			 GDestroyNotify user_data_destroy);

void
bmp_system_control_show_pl_window (BmpSystemControl *self, gboolean show); 

void
bmp_system_control_current_tracklist_set (BmpSystemControl *self, gint _idx); 

void
bmp_system_control_eq_set (BmpSystemControl *self, BmpEqEnum eq, gint value);

void
bmp_system_control_repeat (BmpSystemControl *self, gboolean repeat); 

void
bmp_system_control_shuffle (BmpSystemControl *self, gboolean shuffle); 

void
bmp_system_control_move_start (BmpSystemControl *self);

void
bmp_system_control_move_end (BmpSystemControl *self);

void
bmp_system_control_ui_start (BmpSystemControl *self);

void
bmp_system_control_ui_stop (BmpSystemControl *self);

void
bmp_system_control_quit (BmpSystemControl *self);

void
bmp_system_control_save_state (BmpSystemControl *self);

void
bmp_system_control_bitrate (BmpSystemControl *self, gint bitrate);

void
bmp_system_control_samplerate (BmpSystemControl *self, gint samplerate);

void
bmp_system_control_volume_set (BmpSystemControl *self, gint volume);

void
bmp_system_control_seek (BmpSystemControl *self, gint position);

void
bmp_system_control_play_track (BmpSystemControl *self, gint _idx);

void
bmp_system_control_play_list (BmpSystemControl *self, gint _idx_list); 

void
bmp_system_control_play_list_track (BmpSystemControl *self, gint _idx_list, gint _idx);

void
bmp_system_control_play_pause (BmpSystemControl *self);

gboolean
bmp_system_control_play_stop (BmpSystemControl *self, GError *error);

gboolean
bmp_system_control_play_prev (BmpSystemControl *self, GError *error);

gboolean
bmp_system_control_play_next (BmpSystemControl *self, GError *error);

void
bmp_system_control_play_current (BmpSystemControl *self);

#endif /* BMP_SYSTEM_CONTROL_H */
