/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

/* FIXME: */
#define PATH_MAX 4096

#include <gtk/gtk.h>
#include <fam.h>

#include <fam_monitor.h>

G_DEFINE_TYPE (FamMonitor, fam_monitor, G_TYPE_OBJECT)

typedef enum {
	FM_DELETED,
	FM_CREATED,
	FM_MOVED,

	FM_N_EVENTS
} BmpFamFileEventType;

static gchar *event_names[] = {
	"deleted",
	"created",
	"moved"
};

typedef struct {
	gchar 		    *filename;
	BmpFamFileEventType  event;
} BmpFamFileEventInformation;

typedef enum {
	FAM_MONITOR_SIGNAL_1,
	FAM_MONITOR_N_SIGNALS
} FamMonitorSignals;
static guint fam_monitor_signals[FAM_MONITOR_N_SIGNALS] = {0};

typedef enum {
	FAM_MONITOR_N_PROPERTIES
} FamMonitorProperties;

struct _FamMonitorPrivate
{
        gboolean dispose_has_run;

	gboolean fam_running;
	
	GHashTable *watches;
};

static void
free_fam_watch (FamMonitorWatch *watch)
{
}

static void
fam_monitor_init (FamMonitor *self)
{
	FAMConnection *fc;

        self->private = g_new (FamMonitorPrivate, 1);
        self->private->dispose_has_run = FALSE;
	self->private->watches = g_hash_table_new_full(g_str_hash, g_str_equal,
                                                       (GDestroyNotify) g_free,
                                                       (GDestroyNotify) free_fam_watch);

	fc = g_new0(FAMConnection,1);

	if (FAMOpen(fc) == -1) {
	    self->private->fam_running = FALSE;
	    FAMClose (fc);
	    g_free (fc);
	    g_message ("%s:\t\tINITIALIZED (FAM daemon not running: Not accepting watches!)", __FILE__);
	} else {
	    self->private->fam_running = TRUE;
	    FAMClose (fc);
	    g_free (fc);
	    g_message ("%s:\t\tINITIALIZED", __FILE__);
	}
}

static void
fam_monitor_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static void
fam_monitor_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
        

        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static GObject *
fam_monitor_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                /* Invoke parent constructor. */
                FamMonitorClass *klass;
                GObjectClass *parent_class;  
                klass = FAM_MONITOR_CLASS (g_type_class_peek (FAM_TYPE_MONITOR));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
fam_monitor_dispose (GObject *obj)
{
        FamMonitor *self = (FamMonitor *)obj;

        if (self->private->dispose_has_run)
                return;

        self->private->dispose_has_run = TRUE;

	g_message("%s:\t\tSTOPPED", __FILE__);
}

static void
fam_monitor_finalize (GObject *obj)
{
        FamMonitor *self = (FamMonitor *)obj;

        /*
         * Here, complete object destruction.
         * You might not need to do much...
         */
        g_free (self->private);
}

FamMonitor*
fam_monitor_new (void)
{
  FamMonitor *fam_monitor;

  fam_monitor = g_object_new (fam_monitor_get_type (), NULL); 
 
  return fam_monitor;
}

static void
fam_monitor_class_init (FamMonitorClass *g_class)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
        
        gobject_class->set_property = fam_monitor_set_property;
        gobject_class->get_property = fam_monitor_get_property;
        gobject_class->dispose = fam_monitor_dispose;
        gobject_class->finalize = fam_monitor_finalize;
        gobject_class->constructor = fam_monitor_constructor;
}

void
monitor_thread_func (FamMonitorWatch *fam_watch) {

	FAMEvent *fe = g_new0(FAMEvent,1);
	gint rv;
	gboolean exit_loop = FALSE;
	gboolean req_event = FALSE;

	while (42) {
	
	   rv = FAMPending (fam_watch->fc);

	   switch (rv) {
	   
		case 1: {
		
			BmpFamFileEventInformation *fi;

			FAMNextEvent(fam_watch->fc, fe);

			switch (fe->code) {
			
					case FAMCreated: {
						
							fi = g_new0(BmpFamFileEventInformation,1);	
							fi->filename = g_strdup(fe->filename);
							fi->event = FM_CREATED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					}
					break;
			
					case FAMDeleted: {
						
							fi = g_new0(BmpFamFileEventInformation,1);	
							fi->filename = g_strdup(fe->filename);
							fi->event = FM_DELETED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					}
					break;

					case FAMMoved: {
						
							fi = g_new0(BmpFamFileEventInformation,1);	
							fi->filename = g_strdup(fe->filename);
							fi->event = FM_MOVED;
							fam_watch->fl = g_list_append (fam_watch->fl, fi);
							req_event = TRUE;
					}
					break;

					case FAMStartExecuting: break; 
					case FAMStopExecuting: break; 
					case FAMAcknowledge: break; 
					case FAMChanged: break; 
					case FAMExists: break; 
					case FAMEndExist: break; 
			}
	 	}	
		break;

		case 0: {
		
			exit_loop = TRUE;
	   		g_usleep (50000);
		}
		break;

		default: break;
	   }


	  if (exit_loop && req_event) break;
	  if (fam_watch->terminate) break;
 
	}
	
	g_thread_exit (0);
}

static void
process_fam_events (BmpFamFileEventInformation *fi, FamMonitorWatch *watch) {

	switch (fi->event) {
	
		case FM_CREATED: {
		
			if (watch->added_func)
				watch->added_func(fi->filename, watch->user_data);
		}
		break;

		case FM_DELETED: {
		
			if (watch->removed_func)
				watch->removed_func(fi->filename, watch->user_data);
		}
		break;

    }

    g_free (fi->filename);
}

static gboolean
process_message_queue (FamMonitorWatch *watch) {

	GList *item = NULL;

	item = g_async_queue_try_pop (watch->queue);

	if (!item) return FALSE;

	g_list_foreach (item, (GFunc)process_fam_events, watch);	

	if (g_async_queue_length(watch->queue) == 0) 
		return FALSE;
	else
		return TRUE;
}

static void
monitor_thread (FamMonitorWatch *fam_watch)
{
	GThread *collector;
	GError *error = NULL;

	FAMOpen(fam_watch->fc);
	FAMMonitorDirectory(fam_watch->fc, fam_watch->path_name, fam_watch->fr, NULL);

		while (42) {
		
			collector = 
			    g_thread_create ((GThreadFunc)monitor_thread_func, fam_watch, TRUE, &error); 

			if (error) {
				g_message("%s: %s", G_STRFUNC, error->message);
				g_error_free (error);
				error = NULL;
			}

			g_thread_join (collector);
			g_async_queue_push (fam_watch->queue, g_list_copy(fam_watch->fl));
			g_idle_add ((GSourceFunc) process_message_queue, fam_watch);
			fam_watch->fl = NULL;

			if (fam_watch->terminate) break;
		}

		FAMClose(fam_watch->fc);

		g_thread_exit (0);
}

void
fam_monitor_watch_new (FamMonitor *self, 
		       gchar *monitor_name, 
		       gchar *path_name, 
		       FamAddedFunc added_func, 
		       FamRemovedFunc removed_func, 
		       gpointer user_data, 
		       GDestroyNotify destroy) {

	FamMonitorWatch *fam_watch;
	GError *error = NULL;

	g_return_if_fail (FAM_IS_MONITOR(self));

	if (! self->private->fam_running) {
	    g_message ("%s: Not accepting monitor '%s' for path '%s': FAM daemon not running.", __FILE__, monitor_name, path_name);
	    return;
	}

	fam_watch = g_new0(FamMonitorWatch,1);
	fam_watch->added_func = added_func;
	fam_watch->removed_func = removed_func;
	fam_watch->fc = g_new0(FAMConnection,1);
	fam_watch->fr = g_new0(FAMRequest,1);
	fam_watch->fl = NULL;
	fam_watch->queue = g_async_queue_new();
	fam_watch->terminate = FALSE;
	fam_watch->monitor_name = g_strdup(monitor_name);
	fam_watch->path_name = g_strdup(path_name);
	fam_watch->user_data = user_data;
	fam_watch->destroy = destroy;

	g_hash_table_insert (self->private->watches, g_strdup(monitor_name), fam_watch);	

        fam_watch->thread = g_thread_create ((GThreadFunc)monitor_thread, fam_watch, TRUE, &error); 

	if (error) {
		g_message("%s: %s", G_STRFUNC, error->message);
		g_error_free (error);
		error = NULL;
	}
}

void
fam_monitor_watch_terminate (FamMonitor *self, gchar *monitor_name) {

	FamMonitorWatch *watch;

	watch = g_hash_table_lookup (self->private->watches, monitor_name);

	if (! watch) return;

	watch->terminate = TRUE;
}
