/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */
 
#include <gtk/gtk.h>
#include <list_manager.h>
#include <util.h>

G_DEFINE_TYPE (ListManager, list_manager, G_TYPE_OBJECT)

#if 0
typedef enum {
	LIST_MANAGER_N_SIGNALS
} ListManagerSignals;
static guint list_manager_signals[LIST_MANAGER_N_SIGNALS] = {};
#endif

typedef enum {
	LIST_MANAGER_N_PROPERTIES
} ListManagerProperties;

struct _ListManagerPrivate
{
        gboolean dispose_has_run;

	GHashTable   *lists;
};

static void
list_manager_free_list_item (BmpListType *item)
{
}

static void
list_manager_init (ListManager *self)
{
        self->private = g_new (ListManagerPrivate, 1);
        self->private->dispose_has_run = FALSE;
	self->private->lists = g_hash_table_new_full (g_str_hash, g_str_equal, (GDestroyNotify)g_free, (GDestroyNotify)list_manager_free_list_item); 
}

static void
list_manager_set_property (GObject      *object,
                        guint         property_id,
                        const GValue *value,
                        GParamSpec   *pspec)
{
        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static void
list_manager_get_property (GObject      *object,
                        guint         property_id,
                        GValue       *value,
                        GParamSpec   *pspec)
{
        

        switch (property_id) {

        default:
                /* We don't have any other property... */
                g_assert (FALSE);
                break;
        }
}

static GObject *
list_manager_constructor (GType                  type,
                       guint                  n_construct_properties,
                       GObjectConstructParam *construct_properties)
{
        GObject *obj;

        {
                /* Invoke parent constructor. */
                ListManagerClass *klass;
                GObjectClass *parent_class;  
                klass = LIST_MANAGER_CLASS (g_type_class_peek (LIST_TYPE_MANAGER));
                parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
                obj = parent_class->constructor (type,
                                                 n_construct_properties,
                                                 construct_properties);
        }
        
        /* do stuff. */

        return obj;
}

static void
list_manager_dispose (GObject *obj)
{
        ListManager *self = (ListManager *)obj;

        if (self->private->dispose_has_run) {
                /* If dispose did already run, return. */
                return;
        }
        /* Make sure dispose does not run twice. */
        self->private->dispose_has_run = TRUE;

        /* 
         * In dispose, you are supposed to free all types referenced from this
         * object which might themselves hold a reference to self. Generally,
         * the most simple solution is to unref all members on which you own a 
         * reference.
         */
}

static void
list_manager_finalize (GObject *obj)
{
        ListManager *self = (ListManager *)obj;

        /*
         * Here, complete object destruction.
         * You might not need to do much...
         */
        g_free (self->private);
}

ListManager*
list_manager_new (void)
{
  ListManager *list;
  list = g_object_new (list_manager_get_type (), NULL); 
 
  return list;
}

static void
list_manager_class_init (ListManagerClass *g_class)
{
        GObjectClass *gobject_class = G_OBJECT_CLASS (g_class);
        
        gobject_class->set_property = list_manager_set_property;
        gobject_class->get_property = list_manager_get_property;
        gobject_class->dispose = list_manager_dispose;
        gobject_class->finalize = list_manager_finalize;
        gobject_class->constructor = list_manager_constructor;
}

gint
gint_cmp_func (gpointer a, gpointer b)
{
	gint na,nb;

	na = GPOINTER_TO_INT (a);
	nb = GPOINTER_TO_INT (b);

	return na-nb;
}

void
list_manager_list_rename (ListManager *self, const gchar *name_old, const gchar *name_new)
{
	BmpListType *list;

	list = g_hash_table_lookup (self->private->lists, name_old);
	g_hash_table_steal (self->private->lists, name_old);
	g_hash_table_insert (self->private->lists, g_strdup(name_new), list);	
}

void
list_manager_list_remove (ListManager *self, const gchar *name)
{
    BmpListType *list;

    list = g_hash_table_lookup (self->private->lists, name);

    if (! list) return;

#if 0
    g_list_foreach (list->items, list->item_destroy_func, NULL);

    g_free (list->items);
    g_free (list->name);
#endif

    g_hash_table_remove (self->private->lists, name);
}


BmpListType*
list_manager_list_new (ListManager *self, const gchar *name, 
			ColumnDataFunc column_data_func,
			RowGetSelectedFunc row_get_selected_func,
			RowSetSelectedFunc row_set_selected_func,
			OnDisplayFunc on_display_func,
			ItemSwapFunc item_swap_func,
			ListSortFunc list_sort_func,
			SetDirtyFunc set_dirty_func,
			GetDirtyFunc get_dirty_func,
			SetRenderedFunc set_rendered_func,
			GetRenderedFunc get_rendered_func,
			GDestroyNotify item_destroy_func)

{
	BmpListType *list;	

	list = g_new0(BmpListType,1);

	list->name = g_strdup (name);
	list->column_data_func = column_data_func;
	list->row_get_selected_func = row_get_selected_func;
	list->row_set_selected_func = row_set_selected_func;
	list->on_display_func = on_display_func;
	list->item_swap_func = item_swap_func;
	list->item_destroy_func = item_destroy_func;
	list->list_sort_func = list_sort_func;
	list->set_dirty_func = set_dirty_func;
	list->get_dirty_func = get_dirty_func;
	list->set_rendered_func = set_rendered_func;
	list->get_rendered_func = get_rendered_func;
	list->items = NULL; 
	list->items_shuffle = NULL;
	list->current = -1;
	list->multiple_selected = FALSE;
	list->last_selected = -1;
	list->adjustment = 0.0;

	g_hash_table_insert (self->private->lists, g_strdup(name), list);	

	return list;
}

BmpListType*
list_manager_list_get (ListManager *self, const gchar *name)
{
	BmpListType *item;

	g_return_val_if_fail (LIST_IS_MANAGER (self), NULL);

	item = g_hash_table_lookup (self->private->lists, name);

	if (!item)
		return NULL;
	
	return item;
}

void
list_manager_list_shuffle (ListManager *self, const gchar *name)
{
	BmpListType *item;

	g_return_if_fail (LIST_IS_MANAGER (self));

	item = g_hash_table_lookup (self->private->lists, name);

	if (! item)
		return;

	if (! item->items)
		return;

	item->items_shuffle = glist_make_shuffle_index (item->items);

	item->current = 0; 
	item->current_linear = 0; 
}

void
list_manager_list_sort (ListManager *self, const gchar *name, gpointer data)
{
	GList *llink = NULL;
	BmpListType *list;

	list = list_manager_list_get (self, name);

	if (list->last_selected != -1)
	    llink = 
		g_list_nth (list->items, list->last_selected);

	list->list_sort_func (list, data);

	if (llink)
	    list->last_selected =
		g_list_position (list->items, llink);
}


