/* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*- */

#ifndef LIST_MANAGER_H
#define LIST_MANAGER_H

/*
 * Potentially, include other headers on which this header depends.
 */
#include <glib-object.h>

#define LIST_TYPE_MANAGER		  (list_manager_get_type ())
#define LIST_MANAGER(obj)		  (G_TYPE_CHECK_INSTANCE_CAST ((obj), LIST_TYPE_MANAGER, ListManager))
#define LIST_MANAGER_CLASS(klass)	  (G_TYPE_CHECK_CLASS_CAST ((klass), LIST_TYPE_MANAGER, ListManagerClass))
#define LIST_IS_MANAGER(obj)	  (GTK_CHECK_TYPE ((obj), LIST_TYPE_MANAGER))
#define LIST_IS_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), LIST_TYPE_MANAGER))
#define LIST_MANAGER_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), LIST_TYPE_MANAGER, ListManagerClass))

#define LIST_GET_COLUMN_DATA(list,item,column_idx)\
	list->column_data_func(item,column_idx)

#define LIST_GET_NTH_ITEM(list,row_idx) \
	g_list_nth(list->items,row_idx)->data

#define LIST_GET_NTH_ITEM_GLIST(list,row_idx) \
	g_list_nth(list->items,row_idx)

#define LIST_ADD_ITEM(list,item) \
	list->items = g_list_append(list->items,item)

#define LIST_ADD_ITEM_POSITION(list,item,_idx) \
	list->items = g_list_insert(list->items,item,_idx)

#define LIST_REMOVE_ITEM(list,_idx) \
	list->items = g_list_remove(list->items, g_list_nth(list->items, _idx)->data)

#define LAST_SELECTED_NONE -1

typedef struct _ListManager ListManager;
typedef struct _ListManagerClass ListManagerClass;
typedef struct _ListManagerPrivate ListManagerPrivate;

struct _ListManager {
	GObject parent;

        ListManagerPrivate *private;
};

struct _ListManagerClass {
	GObjectClass parent;
};

typedef gchar*    (*ColumnDataFunc)      (gpointer *item, gint column_idx);
typedef gboolean  (*RowGetSelectedFunc)  (gpointer *item);
typedef void      (*RowSetSelectedFunc)  (gpointer *item, gboolean selected);
typedef void      (*ListSortFunc)        (gconstpointer data1, gconstpointer data2);
typedef void      (*OnDisplayFunc)       (gpointer *item);
typedef void      (*ItemSwapFunc)        (GList    *items, gint _idx1, gint _idx2);
typedef void      (*CurrentSetFunc)      (gpointer *item, gboolean current); 
typedef void      (*SetDirtyFunc)        (gpointer *item, gboolean current); 
typedef gboolean  (*GetDirtyFunc)        (gpointer *item);
typedef void      (*SetRenderedFunc)     (gpointer *item, gboolean rendered); 
typedef gboolean  (*GetRenderedFunc)     (gpointer *item);


typedef struct {
	gchar	    *name;
	GList       *items;
	GList	    *items_shuffle;

	gint         current;
	gint	     current_linear;
	gint	     last_selected;
	gboolean     multiple_selected;

	gdouble	     adjustment;

	ColumnDataFunc     column_data_func;

	RowGetSelectedFunc row_get_selected_func;
	RowSetSelectedFunc row_set_selected_func;
	ListSortFunc	   list_sort_func;
        ItemSwapFunc       item_swap_func;

	OnDisplayFunc	   on_display_func;

	SetDirtyFunc	   set_dirty_func;
	GetDirtyFunc	   get_dirty_func;	

	SetRenderedFunc	   set_rendered_func;
	GetRenderedFunc	   get_rendered_func;	

	GDestroyNotify  item_destroy_func;

} BmpListType;
#define BMP_LIST(list) (BmpListType*)list

/* used by LIST_TYPE_MANAGER */
GType list_manager_get_type (void);

ListManager*
list_manager_new (void);

BmpListType*
list_manager_list_new (ListManager *self, const gchar *name, 
			ColumnDataFunc column_data_func,
			RowGetSelectedFunc row_get_selected_func,
			RowSetSelectedFunc row_set_selected_func,
			OnDisplayFunc on_display_func,
			ItemSwapFunc item_swap_func,
			ListSortFunc list_sort_func,
			SetDirtyFunc set_dirty_func,
			GetDirtyFunc get_dirty_func,
			SetRenderedFunc set_rendered_func,
			GetRenderedFunc get_rendered_func,
			GDestroyNotify item_destroy_func);

BmpListType*
list_manager_list_get (ListManager *self, const gchar *name);

void
list_manager_list_sort (ListManager *self, const gchar *name, gpointer data);

void
list_manager_list_shuffle (ListManager *self, const gchar *name);
	
void
list_manager_list_rename (ListManager *self, const gchar *name_old, const gchar *name_new);

void
list_manager_list_remove (ListManager *self, const gchar *name);
	
#endif /* LIST_MANAGER_H */
