/*  XMMS - Cross-platform multimedia player
 *  Copyright (C) 1998-2003  Peter Alm, Mikael Alm, Olle Hallnas,
 *                           Thomas Nilsson and 4Front Technologies
 *  Copyright (C) 2000-2003  Haavard Kvaalen
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public Licensse as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <config.h>

#include <main.h>
#include <ui_about_dialog.h>

#include <glib.h>
#include <glib/gi18n.h>
#include <gtk/gtk.h>
#include <gdk/gdkkeysyms.h>

static GtkWidget *about_window = NULL;
static GdkPixbuf *about_pixbuf = NULL;

static gboolean
on_about_window_button_press (GtkWidget *widget, GdkEventButton *event, gpointer user_data)
{
	g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);

	if ( (event->button != 1) && (event->type != GDK_BUTTON_PRESS))
		return FALSE;

#if 0
	gdk_window_begin_move_drag (GDK_WINDOW (GTK_WIDGET(widget)->window), event->button, event->x_root, event->y_root, event->time);
#endif

	gtk_widget_hide (widget);
 	return TRUE;
}

static gboolean
on_about_window_key_press (GtkWidget *widget, GdkEventKey *event, gpointer data) {

    g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);

    if (event->keyval == GDK_Escape) {
		gtk_widget_hide (widget);
		return TRUE;
    }

    return FALSE;

}

static gboolean
on_about_window_expose (GtkWidget *widget, GdkEventExpose *event, gpointer data) {

    gdk_draw_pixbuf                 (widget->window,
					     NULL,
                                             about_pixbuf,
                                             0,
                                             0,
                                             0,
                                             0,
                                             -1,
                                             -1,
                                             GDK_RGB_DITHER_MAX,
                                             0,
                                             0);

    return FALSE;
}

void
bmp_about_dialog_raise(void)
{
    if (about_window)
        gdk_window_raise (about_window->window);
}

void
bmp_about_dialog_show(void)
{
    GdkPixmap *mask_pixmap_window1 = NULL, *mask_pixmap_window2 = NULL;
    GdkBitmap *mask_bitmap_window1 = NULL, *mask_bitmap_window2 = NULL;
    GtkWidget *about_logo = NULL;
#if 0
    GtkWidget *close_btn;
    GtkWidget *about_layout;
    GtkWidget *close_image;
#endif
    gchar *filename;

    if (about_window) {
        gtk_window_present(GTK_WINDOW(about_window));
    	gtk_widget_show_all (about_window);
        return;
    }

    about_window = gtk_window_new(GTK_WINDOW_TOPLEVEL);
    gtk_window_set_type_hint(GTK_WINDOW(about_window),
                             GDK_WINDOW_TYPE_HINT_DIALOG);
    filename = DATA_DIR G_DIR_SEPARATOR_S "images" G_DIR_SEPARATOR_S "logo-experimental.png";
    about_logo = gtk_image_new_from_file(filename);
    about_pixbuf = gtk_image_get_pixbuf (GTK_IMAGE (about_logo));

    gtk_widget_set_size_request(GTK_WIDGET(about_window), gdk_pixbuf_get_width(about_pixbuf), gdk_pixbuf_get_height(about_pixbuf));
    gtk_window_set_decorated(GTK_WINDOW(about_window), FALSE);
    gtk_window_set_title(GTK_WINDOW(about_window), Q_("About BMP"));
    gtk_window_set_position(GTK_WINDOW(about_window), GTK_WIN_POS_CENTER);
    gtk_window_set_resizable(GTK_WINDOW(about_window), FALSE);

    gdk_pixbuf_render_pixmap_and_mask (about_pixbuf,
				       &mask_pixmap_window1,
				       &mask_bitmap_window1,
				       0);

    gdk_pixbuf_render_pixmap_and_mask (about_pixbuf,
				       &mask_pixmap_window2,
				       &mask_bitmap_window2,
				       128);

#if 0
    about_layout = gtk_layout_new (NULL, NULL);
    gtk_layout_set_size (GTK_LAYOUT (about_layout), 600, 327);
    gtk_container_add (GTK_CONTAINER (about_window), about_layout);
    gtk_layout_put (GTK_LAYOUT (about_layout), about_logo, 0, 0);

    /* Close button */
    close_btn = gtk_button_new();
    close_image = gtk_image_new_from_stock (GTK_STOCK_CLOSE, GTK_ICON_SIZE_MENU);
    gtk_container_add (GTK_CONTAINER(close_btn), close_image);
    g_signal_connect_swapped(close_btn, "clicked",
                             G_CALLBACK(gtk_widget_destroy), about_window);
    GTK_WIDGET_SET_FLAGS(close_btn, GTK_CAN_DEFAULT);

    g_signal_connect(about_window, "destroy",
                     G_CALLBACK(gtk_widget_destroyed), &about_window);
#endif

#if 0
    GTK_WIDGET_UNSET_FLAGS(about_window, GTK_NO_WINDOW);
#endif

    gtk_widget_add_events (about_window, GDK_ALL_EVENTS_MASK);

    gtk_widget_set_app_paintable (about_window, TRUE);

    g_signal_connect (about_window, "button_press_event", G_CALLBACK(on_about_window_button_press), NULL);
    g_signal_connect (about_window, "key_press_event", G_CALLBACK(on_about_window_key_press), NULL);
    g_signal_connect_after (about_window, "expose_event", G_CALLBACK(on_about_window_expose), NULL);

    gtk_widget_shape_combine_mask (GTK_WIDGET (about_window), mask_bitmap_window2, 0, 0);

    gtk_widget_show_all(about_window);
 }
