#ifndef UTIL_H
#define UTIL_H

#include <gtk/gtk.h>
#include <sys/types.h>


#define NO_PLAY_BUTTON  FALSE
#define PLAY_BUTTON     TRUE

#define SWAP(a, b)      { a^=b; b^=a; a^=b; }


/* file system functions */

typedef gboolean (*DirForeachFunc) (const gchar *path,
                                    const gchar *basename,
                                    gpointer     user_data);

gchar *find_file_recursively (const gchar *dirname,
                              const gchar *file);

gboolean dir_foreach (const gchar     *path,
                      DirForeachFunc   function,
                      gpointer         user_data,
                      GError         **error);

gboolean make_directory (const gchar *path,
                         mode_t       mode);

void del_directory (const gchar *dirname);

gboolean file_is_archive (const gchar * filename);

gchar *archive_decompress (const gchar * path);
gchar *archive_basename   (const gchar * path);



/* INI functions */

gchar *read_ini_string (const gchar *filename,
                        const gchar *section,
                        const gchar *key);

GArray *read_ini_array (const gchar *filename,
                        const gchar *section,
                        const gchar *key);

gchar *read_ini_string_buffer (const gchar *buffer,
                               const gchar *section,
                               const gchar *key);

/* GList functions */

void glist_movedown (GList *list);
void glist_moveup   (GList *list);
void glist_swap     (GList *list_1,
                     GList *list_2);

GList *glist_make_shuffle_index (GList * list);

/* string functions */

gboolean str_has_prefix_nocase   (const gchar * str, const gchar * prefix);
gboolean str_has_suffix_nocase   (const gchar * str, const gchar * suffix);
gboolean str_has_suffixes_nocase (const gchar * str, gchar * const *suffixes);

/* UI functions */

#if ENABLE_NLS
gchar *bmp_menu_translate(const gchar * path, gpointer func_data);
#else
#  define bmp_menu_translate NULL
#endif

void util_set_cursor (GtkWidget * window);

GtkWidget *ui_manager_get_popup (GtkUIManager *self,
                                 const gchar  *path);

void bmp_menu_popup (GtkMenu *menu,
                     gint     x,
                     gint     y,
                     guint    button,
                     guint    time);

gboolean bmp_become_daemon (void);

gchar *bmp_get_titlestring (GHashTable *metadata);

gchar *bmp_get_file_http (const gchar  *uri,
                          GError *error);

/* Playlist parsers */

GList *get_uris__pls      (const gchar *uri);
GList *get_uris__m3u      (const gchar *uri);
GList *get_uris__playlist (const gchar *uri);


/* Miscellaneous */

gboolean collect_path (const gchar  *path,
                       const gchar  *basename,
                       GList       **items);

gboolean is_audio_file (const gchar *filename);

guint gint_count_digits (gint n);

#endif
